/**
 * 用于维护所有的 波形容器视图
 * 设计参考文档：https://nc-ai-lab.feishu.cn/wiki/Fy3ZwtbYbiatmxkhOp2cyHSFnlw
 */

import { globalLookup } from "./global";
import { getSmartCurrentSignalValue } from "./utils";


/**
 * @namespace WaveContainerView
 */
export const WaveContainerView = {
    /**
     * 
     * @param {WireItem} signal 
     */
    add(signal) {
        // 增加高效 CRUD 视图
        globalLookup.currentWires.add(signal);

        // 上面的 视图 的 string, signal 字典版本
        globalLookup.link2CurrentWires.set(signal.link, signal);

        // 增加 sidebar 渲染视图
        // TODO : 支持更加复杂的视图加入
        globalLookup.currentWiresRenderView.push({
            signalInfo: {
                name: signal.name,
                link: signal.link,
                size: signal.size,
            },
            groupInfo: {
                name: '',
                color: '',
                collapse: false
            },
            renderType: 0,
            children: []
        });

        // 增加当前波形值视图
        const changoItem = globalLookup.chango[signal.link];
        
        globalLookup.currentSignalValues[signal.link] = getSmartCurrentSignalValue(changoItem, globalLookup.currentTime);
    },

    /**
     * 
     * @param {WireItem} signal 
     */
    delete(signal) {
        const renderView = globalLookup.currentWiresRenderView;

        globalLookup.currentWires.delete(signal);
        globalLookup.link2CurrentWires.delete(signal.link);
        delete globalLookup.currentSignalValues[signal.link];
        
        // 从 currentWiresRenderView 中删除
        const i = findViewIndexByLink(signal.link);
        const tailElements = renderView.slice(i + 1);
        renderView.length = i;
        tailElements.forEach(view => renderView.push(view));

    },

    /**
     * 
     * @param {WireItem} signal 
     * @returns {boolean}
     */
    has(signal) {
        return globalLookup.currentWires.has(signal);
    }
};


export function findViewIndexByLink(link) {
    const renderView = globalLookup.currentWiresRenderView;

    let i = 0;
    while (renderView[i].signalInfo.link !== link && i < renderView.length) {
        ++ i;
    }

    if (i < renderView.length) {
        return i;
    } else {
        return -1;
    }
}


function makeFullSignalNameDeps(signal) {
    const deps = [];
    while (signal) {
        if (signal.name && signal.type) {
            deps.push(signal);
        }
        signal = signal.parent;
    }
    let htmlString = '';
    for (let i = deps.length - 1; i >= 0; -- i) {
        const mod = deps[i];
        // const displayName = mod.name.length > 6 ? mod.name.substring(0, 6) + '...' : mod.name;
        const iconClass = makeIconClass(mod);
        const iconText = `<span class="iconfont ${iconClass}"></span>&ensp;${mod.name}`;
        
        htmlString += iconText;

        if (i > 0) {
            htmlString += '<div class="dep-arrow"></div>';
        }
    }
    htmlString = '<div class="signal-info-tooltip-wrapper">' + htmlString + '</div>';
    return htmlString;
}